package com.devteria.identityservice.service;

import com.devteria.identityservice.enums.OtpType;
import lombok.AccessLevel;
import lombok.RequiredArgsConstructor;
import lombok.experimental.FieldDefaults;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.security.SecureRandom;
import java.util.concurrent.TimeUnit;

@Service
@RequiredArgsConstructor
@Slf4j
@FieldDefaults(level = AccessLevel.PRIVATE, makeFinal = true)
public class OtpService {
    static final String OTP_PREFIX = "otp:";
    static final int OTP_LENGTH = 6;
    RedisTemplate<String, Object> redisTemplate;


    public String generateAndSaveOtp(String identifier, OtpType otpType) {
        String otp = generateOtp(OTP_LENGTH);
        String key = OTP_PREFIX + otpType.getName() + identifier;

        redisTemplate.opsForValue().set(key, otp, otpType.getExpireTimeInSeconds(), TimeUnit.SECONDS);
        log.info("Generated OTP for {}: {} with type {} (expires in {} seconds)",
                identifier, key, otpType, otpType.getExpireTimeInSeconds());
        return otp;
    }

    public boolean validateOtp(String otp, String identifier, OtpType otpType) {
        String key = OTP_PREFIX + otpType.getName() + identifier;
        String storedOtp = (String) redisTemplate.opsForValue().get(key);

        if (storedOtp != null && storedOtp.equals(otp)) {
            // Delete OTP after successful validation
            redisTemplate.delete(key);
            return true;
        }

        return false;
    }

    private String generateOtp(int length) {
        SecureRandom random = new SecureRandom();
        StringBuilder otp = new StringBuilder();

        for (int i = 0; i < length; i++) {
            otp.append(random.nextInt(10));
        }

        return otp.toString();
    }

}
